@@ -1,6 +1,3 @@
 from lightwood.encoders.categorical.onehot import OneHotEncoder
+from lightwood.encoders.categorical.multihot import MultiHotEncoder
 from lightwood.encoders.categorical.autoencoder import CategoricalAutoEncoder
-
-default = CategoricalAutoEncoder
-onehot = OneHotEncoder
-autoencoder = CategoricalAutoEncoder
